/* Copyright (c) 2012 Nordic Semiconductor. All Rights Reserved.
*
* The information contained herein is property of Nordic Semiconductor ASA.
* Terms and conditions of usage are described in detail in NORDIC
* SEMICONDUCTOR STANDARD SOFTWARE LICENSE AGREEMENT.
*
* Licensees are granted free, non-transferable use of the information. NO
* WARRANTY of ANY KIND is provided. This heading must NOT be removed from
* the file.
*
* $LastChangedRevision: 15516 $
*/

/** 
 * @file
 * @brief Gazell Link Layer Host with Payload in ACK example
 * @defgroup gzll_host_ack_payload_example Gazell Link Layer Host with Payload in ACK
 * @{
 * @ingroup gzll_03_examples
 *
 * This project requires that a Device running the 
 * @ref gzll_device_ack_payload_example be used as a counterpart for 
 * receiving the data. This can be on either nRF51 device or a nRF24Lxx device
 * running the \b gzll_device_ack_payload example in the nRFgo SDK. 
 * 
 * This example listens for a packet and sends an ACK
 * when a packet is received. The contents of the first payload byte of 
 * the received packet is output on the GPIO Port BUTTONS. 
 * The contents of GPIO Port LEDS are sent in the first payload byte (byte 0) 
 * of the ACK packet.
 */


#include "nrf_gzll.h"
#include "nrf_gpio.h"
#include "nrf_delay.h"
#include "simple_uart.h"
#include "app_timer.h"
#include "boards.h"
#include <stdbool.h>
#include <stdio.h>

// App timer defines
#define APP_TIMER_PRESCALER         0                                           /**< Value of the RTC1 PRESCALER register. */
#define APP_TIMER_MAX_TIMERS        5 // One instance only.
#define APP_TIMER_OP_QUEUE_SIZE     5 // Maximum number of timeout handlers pending execution
#define APP_TIMER_UART_INTERVAL     1000

/*****************************************************************************/
/** @name Configuration  */
/*****************************************************************************/

// Define pipe
#define PIPE_NUMBER 0 ///< We use pipe 0 in this example

// Define payload length
#define TX_PAYLOAD_LENGTH 32
#define RX_PAYLOAD_LENGTH 32

// Data and acknowledgement payloads
static uint8_t              data_payload[RX_PAYLOAD_LENGTH];  ///< Placeholder for data payload received from host. 
volatile uint32_t           bytes_received;
static app_timer_id_t       radio_max_throughput_send_timer_id;

// Implementation of sendchar (also used by printf function to output data)
int sendchar (int ch)  {                 // Write character to Serial Port
  simple_uart_put((uint8_t)ch);
  return ch;
}

int getkey (void)  {                     // Read character from Serial Port

  return (int)simple_uart_get();
}

// Dummy handler needed by app_timer.
void app_error_handler(uint32_t error_code, uint32_t line_num, const uint8_t * p_file_name)
{
    while(1);
}

void timers_init(void)
{
    // Start the 32KHz first
    NRF_CLOCK->LFCLKSRC = (CLOCK_LFCLKSRC_SRC_Xtal << CLOCK_LFCLKSRC_SRC_Pos);    
    NRF_CLOCK->TASKS_LFCLKSTART = 1;
    while (NRF_CLOCK->EVENTS_LFCLKSTARTED == 0);
    NRF_CLOCK->EVENTS_LFCLKSTARTED = 0;

    // Initialize timer module, no scheduler
    APP_TIMER_INIT(APP_TIMER_PRESCALER, APP_TIMER_MAX_TIMERS, APP_TIMER_OP_QUEUE_SIZE, false);
}

void uart_send_callback(void *p_context)
{
#if 0
    static uint32_t prev_bc;
    // this will give bytes per second
    printf("\n\rThroughput in bytes: %d", (bytes_received-prev_bc));
    
    // This will give kiloBits per second, rounded down (shifted left 3 times gives bits, shift right 10 (div 1024) gives kBits).
    //printf("\n\rThroughput in kbits: %d", (bytes_received-prev_bc) >> 7);
    prev_bc = bytes_received;
#endif    
}

void timers_start(void)
{
    // Allocate space
    (void)app_timer_create(&radio_max_throughput_send_timer_id, APP_TIMER_MODE_REPEATED, uart_send_callback);
    // Start the task, 1 second interval to get KBps.
    (void)app_timer_start(radio_max_throughput_send_timer_id, APP_TIMER_TICKS(APP_TIMER_UART_INTERVAL, APP_TIMER_PRESCALER), 0);    
}

/** @} */

/*****************************************************************************/
/**
* @brief Main function.
* @return ANSI required int return type.
*/
/*****************************************************************************/
int main()
{     
    // Initialize Gazell
    (void)nrf_gzll_init(NRF_GZLL_MODE_HOST);
    //(void)nrf_gzll_set_base_address_0(0xE7E7E7E7);
    // Enable Gazell to start sending over the air
    (void)nrf_gzll_enable();
    simple_uart_config(RTS_PIN_NUMBER, TX_PIN_NUMBER, CTS_PIN_NUMBER, RX_PIN_NUMBER, HWFC);
    printf("Welcome to gzll maxrate test!\n\r");
    timers_init();
    timers_start();
    while(1)
    {
    }
}


/*****************************************************************************/
/** @name Gazell callback function definitions  */
/*****************************************************************************/


// If a data packet was received, we write the first byte to LEDS. 
void nrf_gzll_host_rx_data_ready(uint32_t pipe, nrf_gzll_host_rx_info_t rx_info)
{   
    uint32_t data_payload_length = RX_PAYLOAD_LENGTH;
    bool pop_ok = false;
    printf("\n\rRSSI : %d", (rx_info.rssi));
    // Pop packet and write first byte of the payload to the GPIO port.
    pop_ok = nrf_gzll_fetch_packet_from_rx_fifo(pipe, data_payload, &data_payload_length);
    if (pop_ok)
        bytes_received += data_payload_length;
}

// Callbacks not needed in this example.
void nrf_gzll_device_tx_success(uint32_t pipe, nrf_gzll_device_tx_info_t tx_info) {}
void nrf_gzll_device_tx_failed(uint32_t pipe, nrf_gzll_device_tx_info_t tx_info) {}
void nrf_gzll_disabled() {}

/** @} */
/** @} */

